"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertAccountNode = exports.isAccountNode = exports.accountNodeFromIdl = exports.accountNode = void 0;
const shared_1 = require("../shared");
const AccountDataNode_1 = require("./AccountDataNode");
const BytesTypeNode_1 = require("./BytesTypeNode");
const StringTypeNode_1 = require("./StringTypeNode");
const StructTypeNode_1 = require("./StructTypeNode");
const TypeNode_1 = require("./TypeNode");
const ValueNode_1 = require("./ValueNode");
function accountNode(input) {
    var _a, _b, _c, _d;
    if (!input.name) {
        throw new shared_1.InvalidKinobiTreeError('AccountNode must have a name.');
    }
    return {
        kind: 'accountNode',
        name: (0, shared_1.mainCase)(input.name),
        data: input.data,
        idlName: (_a = input.idlName) !== null && _a !== void 0 ? _a : input.name,
        docs: (_b = input.docs) !== null && _b !== void 0 ? _b : [],
        internal: (_c = input.internal) !== null && _c !== void 0 ? _c : false,
        size: input.size,
        seeds: (_d = input.seeds) !== null && _d !== void 0 ? _d : [],
        discriminator: input.discriminator,
    };
}
exports.accountNode = accountNode;
function accountNodeFromIdl(idl) {
    var _a, _b, _c, _d;
    const idlName = (_a = idl.name) !== null && _a !== void 0 ? _a : '';
    const name = (0, shared_1.mainCase)(idlName);
    const idlStruct = (_b = idl.type) !== null && _b !== void 0 ? _b : { kind: 'struct', fields: [] };
    const struct = (0, TypeNode_1.createTypeNodeFromIdl)(idlStruct);
    (0, StructTypeNode_1.assertStructTypeNode)(struct);
    const seeds = ((_c = idl.seeds) !== null && _c !== void 0 ? _c : []).map((seed) => {
        if (seed.kind === 'constant') {
            const value = (0, ValueNode_1.vScalar)(seed.value);
            let type;
            if (seed.type === 'string') {
                type = (0, StringTypeNode_1.stringTypeNode)({ size: (0, shared_1.remainderSize)() });
            }
            else if (seed.type === 'bytes') {
                type = (0, BytesTypeNode_1.bytesTypeNode)((0, shared_1.remainderSize)());
            }
            else {
                type = (0, TypeNode_1.createTypeNodeFromIdl)(seed.type);
            }
            return { ...seed, type, value };
        }
        if (seed.kind === 'variable') {
            return {
                ...seed,
                type: (0, TypeNode_1.createTypeNodeFromIdl)(seed.type),
                docs: seed.description ? [seed.description] : [],
            };
        }
        return { kind: 'programId' };
    });
    return accountNode({
        name,
        data: (0, AccountDataNode_1.accountDataNode)({ name: `${name}AccountData`, struct }),
        idlName,
        docs: (_d = idl.docs) !== null && _d !== void 0 ? _d : [],
        size: idl.size,
        seeds,
    });
}
exports.accountNodeFromIdl = accountNodeFromIdl;
function isAccountNode(node) {
    return !!node && node.kind === 'accountNode';
}
exports.isAccountNode = isAccountNode;
function assertAccountNode(node) {
    var _a;
    if (!isAccountNode(node)) {
        throw new Error(`Expected accountNode, got ${(_a = node === null || node === void 0 ? void 0 : node.kind) !== null && _a !== void 0 ? _a : 'null'}.`);
    }
}
exports.assertAccountNode = assertAccountNode;
//# sourceMappingURL=AccountNode.js.map