"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertInstructionNode = exports.isInstructionNode = exports.getAllInstructionsWithSubs = exports.instructionNodeFromIdl = exports.instructionNode = void 0;
const shared_1 = require("../shared");
const InstructionAccountNode_1 = require("./InstructionAccountNode");
const InstructionDataArgsNode_1 = require("./InstructionDataArgsNode");
const InstructionExtraArgsNode_1 = require("./InstructionExtraArgsNode");
const ProgramNode_1 = require("./ProgramNode");
const StructFieldTypeNode_1 = require("./StructFieldTypeNode");
const StructTypeNode_1 = require("./StructTypeNode");
const TypeNode_1 = require("./TypeNode");
const ValueNode_1 = require("./ValueNode");
function instructionNode(input) {
    var _a, _b, _c, _d, _e, _f;
    if (!input.name) {
        throw new shared_1.InvalidKinobiTreeError('InstructionNode must have a name.');
    }
    const name = (0, shared_1.mainCase)(input.name);
    return {
        kind: 'instructionNode',
        name,
        accounts: input.accounts,
        dataArgs: input.dataArgs,
        extraArgs: (_a = input.extraArgs) !== null && _a !== void 0 ? _a : (0, InstructionExtraArgsNode_1.instructionExtraArgsNode)({
            name: `${name}InstructionExtra`,
            struct: (0, StructTypeNode_1.structTypeNode)([]),
        }),
        subInstructions: (_b = input.subInstructions) !== null && _b !== void 0 ? _b : [],
        idlName: (_c = input.idlName) !== null && _c !== void 0 ? _c : input.name,
        docs: (_d = input.docs) !== null && _d !== void 0 ? _d : [],
        internal: (_e = input.internal) !== null && _e !== void 0 ? _e : false,
        bytesCreatedOnChain: input.bytesCreatedOnChain,
        argDefaults: Object.fromEntries(Object.entries((_f = input.argDefaults) !== null && _f !== void 0 ? _f : {}).map(([key, value]) => [
            (0, shared_1.mainCase)(key),
            value,
        ])),
    };
}
exports.instructionNode = instructionNode;
function instructionNodeFromIdl(idl) {
    var _a, _b, _c, _d;
    const idlName = (_a = idl.name) !== null && _a !== void 0 ? _a : '';
    const name = (0, shared_1.mainCase)(idlName);
    const useProgramIdForOptionalAccounts = !idl.legacyOptionalAccountsStrategy;
    let dataArgs = (0, StructTypeNode_1.structTypeNodeFromIdl)({
        kind: 'struct',
        fields: (_b = idl.args) !== null && _b !== void 0 ? _b : [],
    });
    if (idl.discriminant) {
        const discriminatorField = (0, StructFieldTypeNode_1.structFieldTypeNode)({
            name: 'discriminator',
            child: (0, TypeNode_1.createTypeNodeFromIdl)(idl.discriminant.type),
            defaultsTo: {
                strategy: 'omitted',
                value: (0, ValueNode_1.vScalar)(idl.discriminant.value),
            },
        });
        dataArgs = (0, StructTypeNode_1.structTypeNode)([discriminatorField, ...dataArgs.fields]);
    }
    return instructionNode({
        name,
        idlName,
        docs: (_c = idl.docs) !== null && _c !== void 0 ? _c : [],
        accounts: ((_d = idl.accounts) !== null && _d !== void 0 ? _d : []).map((account) => (0, InstructionAccountNode_1.instructionAccountNodeFromIdl)(account, useProgramIdForOptionalAccounts)),
        dataArgs: (0, InstructionDataArgsNode_1.instructionDataArgsNode)({
            name: `${name}InstructionData`,
            struct: dataArgs,
        }),
    });
}
exports.instructionNodeFromIdl = instructionNodeFromIdl;
function getAllInstructionsWithSubs(node, leavesOnly = false) {
    if (isInstructionNode(node)) {
        if (node.subInstructions.length === 0)
            return [node];
        const subInstructions = node.subInstructions.flatMap((sub) => getAllInstructionsWithSubs(sub, leavesOnly));
        return leavesOnly ? subInstructions : [node, ...subInstructions];
    }
    const instructions = (0, ProgramNode_1.isProgramNode)(node)
        ? node.instructions
        : node.programs.flatMap((program) => program.instructions);
    return instructions.flatMap((instruction) => getAllInstructionsWithSubs(instruction, leavesOnly));
}
exports.getAllInstructionsWithSubs = getAllInstructionsWithSubs;
function isInstructionNode(node) {
    return !!node && node.kind === 'instructionNode';
}
exports.isInstructionNode = isInstructionNode;
function assertInstructionNode(node) {
    var _a;
    if (!isInstructionNode(node)) {
        throw new Error(`Expected instructionNode, got ${(_a = node === null || node === void 0 ? void 0 : node.kind) !== null && _a !== void 0 ? _a : 'null'}.`);
    }
}
exports.assertInstructionNode = assertInstructionNode;
//# sourceMappingURL=InstructionNode.js.map