"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertMapTypeNode = exports.isMapTypeNode = exports.mapTypeNodeFromIdl = exports.mapTypeNode = void 0;
const shared_1 = require("../shared");
const NumberTypeNode_1 = require("./NumberTypeNode");
const TypeNode_1 = require("./TypeNode");
function mapTypeNode(key, value, options = {}) {
    var _a, _b;
    return {
        kind: 'mapTypeNode',
        key,
        value,
        size: (_a = options.size) !== null && _a !== void 0 ? _a : (0, shared_1.prefixedSize)(),
        idlMap: (_b = options.idlMap) !== null && _b !== void 0 ? _b : 'hashMap',
    };
}
exports.mapTypeNode = mapTypeNode;
function mapTypeNodeFromIdl(idl) {
    const [key, value] = 'hashMap' in idl ? idl.hashMap : idl.bTreeMap;
    let size;
    if (idl.size === 'remainder') {
        size = (0, shared_1.remainderSize)();
    }
    else if (typeof idl.size === 'number') {
        size = (0, shared_1.fixedSize)(idl.size);
    }
    else if (idl.size) {
        size = (0, shared_1.prefixedSize)((0, NumberTypeNode_1.numberTypeNode)(idl.size));
    }
    return mapTypeNode((0, TypeNode_1.createTypeNodeFromIdl)(key), (0, TypeNode_1.createTypeNodeFromIdl)(value), {
        size,
        idlMap: 'hashMap' in idl ? 'hashMap' : 'bTreeMap',
    });
}
exports.mapTypeNodeFromIdl = mapTypeNodeFromIdl;
function isMapTypeNode(node) {
    return !!node && node.kind === 'mapTypeNode';
}
exports.isMapTypeNode = isMapTypeNode;
function assertMapTypeNode(node) {
    var _a;
    if (!isMapTypeNode(node)) {
        throw new Error(`Expected mapTypeNode, got ${(_a = node === null || node === void 0 ? void 0 : node.kind) !== null && _a !== void 0 ? _a : 'null'}.`);
    }
}
exports.assertMapTypeNode = assertMapTypeNode;
//# sourceMappingURL=MapTypeNode.js.map