"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JavaScriptImportMap = void 0;
const DEFAULT_MODULE_MAP = {
    umi: '@metaplex-foundation/umi',
    umiSerializers: '@metaplex-foundation/umi/serializers',
    types: '../types',
    errors: '../errors',
    shared: '../shared',
};
class JavaScriptImportMap {
    constructor() {
        this._imports = new Map();
        this._aliases = new Map();
    }
    add(module, imports) {
        var _a;
        const currentImports = (_a = this._imports.get(module)) !== null && _a !== void 0 ? _a : new Set();
        const newImports = typeof imports === 'string' ? [imports] : imports;
        newImports.forEach((i) => currentImports.add(i));
        this._imports.set(module, currentImports);
        return this;
    }
    remove(module, imports) {
        var _a;
        const currentImports = (_a = this._imports.get(module)) !== null && _a !== void 0 ? _a : new Set();
        const importsToRemove = typeof imports === 'string' ? [imports] : imports;
        importsToRemove.forEach((i) => currentImports.delete(i));
        if (currentImports.size === 0) {
            this._imports.delete(module);
        }
        else {
            this._imports.set(module, currentImports);
        }
        return this;
    }
    mergeWith(...others) {
        others.forEach((other) => {
            other._imports.forEach((imports, module) => {
                this.add(module, imports);
            });
            other._aliases.forEach((aliases, module) => {
                Object.entries(aliases).forEach(([name, alias]) => {
                    this.addAlias(module, name, alias);
                });
            });
        });
        return this;
    }
    mergeWithManifest(manifest) {
        return this.mergeWith(manifest.strictImports, manifest.looseImports, manifest.serializerImports);
    }
    addAlias(module, name, alias) {
        var _a;
        const currentAliases = (_a = this._aliases.get(module)) !== null && _a !== void 0 ? _a : {};
        currentAliases[name] = alias;
        this._aliases.set(module, currentAliases);
        return this;
    }
    isEmpty() {
        return this._imports.size === 0;
    }
    toString(dependencies) {
        const dependencyMap = { ...DEFAULT_MODULE_MAP, ...dependencies };
        const importStatements = [...this._imports.entries()]
            .map(([module, imports]) => {
            var _a;
            const mappedModule = (_a = dependencyMap[module]) !== null && _a !== void 0 ? _a : module;
            return [mappedModule, module, imports];
        })
            .sort(([a], [b]) => {
            const aIsRelative = a.startsWith('.');
            const bIsRelative = b.startsWith('.');
            if (aIsRelative && !bIsRelative)
                return 1;
            if (!aIsRelative && bIsRelative)
                return -1;
            return a.localeCompare(b);
        })
            .map(([mappedModule, module, imports]) => {
            var _a;
            const aliasMap = (_a = this._aliases.get(module)) !== null && _a !== void 0 ? _a : {};
            const joinedImports = [...imports]
                .sort()
                .map((i) => (aliasMap[i] ? `${i} as ${aliasMap[i]}` : i))
                .join(', ');
            return `import { ${joinedImports} } from '${mappedModule}';`;
        });
        return importStatements.join('\n');
    }
}
exports.JavaScriptImportMap = JavaScriptImportMap;
//# sourceMappingURL=JavaScriptImportMap.js.map