"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetByteSizeVisitor = void 0;
const nodes = __importStar(require("../../nodes"));
const BaseThrowVisitor_1 = require("../BaseThrowVisitor");
const Visitor_1 = require("../Visitor");
class GetByteSizeVisitor extends BaseThrowVisitor_1.BaseThrowVisitor {
    constructor() {
        super(...arguments);
        this.availableDefinedTypes = new Map();
        this.visitedDefinedTypes = new Map();
        this.definedTypeStack = [];
    }
    registerDefinedTypes(definedTypes) {
        definedTypes.forEach((definedType) => {
            this.availableDefinedTypes.set(definedType.name, definedType);
        });
    }
    visitAccount(account) {
        return (0, Visitor_1.visit)(account.data, this);
    }
    visitAccountData(accountData) {
        return (0, Visitor_1.visit)(accountData.struct, this);
    }
    visitInstruction(instruction) {
        return (0, Visitor_1.visit)(instruction.dataArgs, this);
    }
    visitInstructionDataArgs(instructionDataArgs) {
        return (0, Visitor_1.visit)(instructionDataArgs.struct, this);
    }
    visitInstructionExtraArgs(instructionExtraArgs) {
        return (0, Visitor_1.visit)(instructionExtraArgs.struct, this);
    }
    visitDefinedType(definedType) {
        if (this.visitedDefinedTypes.has(definedType.name)) {
            return this.visitedDefinedTypes.get(definedType.name);
        }
        this.definedTypeStack.push(definedType.name);
        const child = (0, Visitor_1.visit)(definedType.data, this);
        this.definedTypeStack.pop();
        this.visitedDefinedTypes.set(definedType.name, child);
        return child;
    }
    visitArrayType(arrayType) {
        if (arrayType.size.kind !== 'fixed')
            return null;
        const fixedSize = arrayType.size.value;
        const childSize = (0, Visitor_1.visit)(arrayType.child, this);
        const arraySize = childSize !== null ? childSize * fixedSize : null;
        return fixedSize === 0 ? 0 : arraySize;
    }
    visitLinkType(linkType) {
        if (linkType.size !== undefined)
            return linkType.size;
        if (linkType.importFrom !== 'generated')
            return null;
        // Fetch the linked type and return null if not found.
        // The validator visitor will throw a proper error later on.
        const linkedDefinedType = this.availableDefinedTypes.get(linkType.name);
        if (!linkedDefinedType) {
            return null;
        }
        // This prevents infinite recursion by using assuming
        // cyclic types don't have a fixed size.
        if (this.definedTypeStack.includes(linkedDefinedType.name)) {
            return null;
        }
        return (0, Visitor_1.visit)(linkedDefinedType, this);
    }
    visitEnumType(enumType) {
        var _a;
        const prefix = (_a = (0, Visitor_1.visit)(enumType.size, this)) !== null && _a !== void 0 ? _a : 1;
        if (nodes.isScalarEnum(enumType))
            return prefix;
        const variantSizes = enumType.variants.map((v) => (0, Visitor_1.visit)(v, this));
        const allVariantHaveTheSameFixedSize = variantSizes.every((one, i, all) => one === all[0]);
        return allVariantHaveTheSameFixedSize &&
            variantSizes.length > 0 &&
            variantSizes[0] !== null
            ? variantSizes[0] + prefix
            : null;
    }
    visitEnumEmptyVariantType() {
        return 0;
    }
    visitEnumStructVariantType(enumStructVariantType) {
        return (0, Visitor_1.visit)(enumStructVariantType.struct, this);
    }
    visitEnumTupleVariantType(enumTupleVariantType) {
        return (0, Visitor_1.visit)(enumTupleVariantType.tuple, this);
    }
    visitMapType() {
        return null;
    }
    visitOptionType(optionType) {
        if (!optionType.fixed)
            return null;
        const prefixSize = (0, Visitor_1.visit)(optionType.prefix, this);
        const childSize = (0, Visitor_1.visit)(optionType.child, this);
        return childSize !== null ? childSize + prefixSize : null;
    }
    visitSetType() {
        return null;
    }
    visitStructType(structType) {
        return this.sumSizes(structType.fields.map((f) => (0, Visitor_1.visit)(f, this)));
    }
    visitStructFieldType(structFieldType) {
        return (0, Visitor_1.visit)(structFieldType.child, this);
    }
    visitTupleType(tupleType) {
        return this.sumSizes(tupleType.children.map((child) => (0, Visitor_1.visit)(child, this)));
    }
    visitBoolType(boolType) {
        return (0, Visitor_1.visit)(boolType.size, this);
    }
    visitBytesType(bytesType) {
        if (bytesType.size.kind !== 'fixed')
            return null;
        return bytesType.size.value;
    }
    visitNumberType(numberType) {
        return parseInt(numberType.format.slice(1), 10) / 8;
    }
    visitNumberWrapperType(numberWrapperType) {
        return (0, Visitor_1.visit)(numberWrapperType.number, this);
    }
    visitPublicKeyType() {
        return 32;
    }
    visitStringType(stringType) {
        if (stringType.size.kind !== 'fixed')
            return null;
        return stringType.size.value;
    }
    sumSizes(sizes) {
        return sizes.reduce((all, one) => (all === null || one === null ? null : all + one), 0);
    }
}
exports.GetByteSizeVisitor = GetByteSizeVisitor;
//# sourceMappingURL=GetByteSizeVisitor.js.map