"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetNodeTreeStringVisitor = void 0;
const shared_1 = require("../../shared");
const nodes = __importStar(require("../../nodes"));
const Visitor_1 = require("../Visitor");
class GetNodeTreeStringVisitor {
    constructor(separator = '|   ') {
        this.indent = 0;
        this.separator = separator;
    }
    visitRoot(root) {
        this.indent += 1;
        const children = root.programs.map((program) => (0, Visitor_1.visit)(program, this));
        this.indent -= 1;
        return [this.indented('[RootNode]'), ...children].join('\n');
    }
    visitProgram(program) {
        this.indent += 1;
        const children = [
            ...program.accounts.map((account) => (0, Visitor_1.visit)(account, this)),
            ...nodes.getAllInstructionsWithSubs(program).map((ix) => (0, Visitor_1.visit)(ix, this)),
            ...program.definedTypes.map((type) => (0, Visitor_1.visit)(type, this)),
            ...program.errors.map((type) => (0, Visitor_1.visit)(type, this)),
        ];
        this.indent -= 1;
        const tags = [];
        if (program.publicKey)
            tags.push(`publicKey: ${program.publicKey}`);
        if (program.version)
            tags.push(`version: ${program.version}`);
        if (program.origin)
            tags.push(`origin: ${program.origin}`);
        const tagsStr = tags.length > 0 ? ` (${tags.join(', ')})` : '';
        return [
            this.indented(`[ProgramNode] ${program.name}${tagsStr}`),
            ...children,
        ].join('\n');
    }
    visitAccount(account) {
        const children = [];
        children.push(this.indented(`[AccountNode] ${account.name}`));
        this.indent += 1;
        if (account.size !== undefined) {
            children.push(this.indented(`size: ${account.size}`));
        }
        if (account.seeds.length > 0) {
            children.push(this.indented('seeds:'));
            this.indent += 1;
            children.push(...account.seeds.map((seed) => {
                if (seed.kind === 'programId')
                    return this.indented('programId');
                if (seed.kind === 'constant') {
                    if (seed.value.kind === 'string') {
                        return this.indented(`"${seed.value.value}"`);
                    }
                    if (seed.value.kind === 'number' || seed.value.kind === 'boolean') {
                        return this.indented(`${seed.value.value}`);
                    }
                    return this.indented(`[${seed.value.kind}]`);
                }
                this.indent += 1;
                const type = (0, Visitor_1.visit)(seed.type, this);
                this.indent -= 1;
                return [
                    this.indented(`${seed.name} (${seed.docs.join(' ')})`),
                    type,
                ].join('\n');
            }));
            this.indent -= 1;
        }
        children.push((0, Visitor_1.visit)(account.data, this));
        this.indent -= 1;
        return children.join('\n');
    }
    visitAccountData(accountData) {
        const children = [];
        this.indent += 1;
        children.push((0, Visitor_1.visit)(accountData.struct, this));
        if (accountData.link) {
            children.push((0, Visitor_1.visit)(accountData.link, this));
        }
        this.indent -= 1;
        return [this.indented('[AccountDataNode]'), ...children].join('\n');
    }
    visitInstruction(instruction) {
        const children = [];
        children.push(this.indented(`[InstructionNode] ${instruction.name}`));
        this.indent += 1;
        children.push(this.indented('accounts:'));
        this.indent += 1;
        children.push(...instruction.accounts.map((account) => (0, Visitor_1.visit)(account, this)));
        this.indent -= 1;
        children.push((0, Visitor_1.visit)(instruction.dataArgs, this));
        children.push((0, Visitor_1.visit)(instruction.extraArgs, this));
        this.indent -= 1;
        return children.join('\n');
    }
    visitInstructionAccount(instructionAccount) {
        const tags = [];
        if (instructionAccount.isWritable)
            tags.push('writable');
        if (instructionAccount.isSigner)
            tags.push('signer');
        if (instructionAccount.isOptional)
            tags.push('optional');
        if (instructionAccount.defaultsTo)
            tags.push(`defaults to ${instructionAccount.defaultsTo.kind}`);
        const tagsAsString = tags.length > 0 ? ` (${tags.join(', ')})` : '';
        return this.indented(instructionAccount.name + tagsAsString);
    }
    visitInstructionDataArgs(instructionDataArgs) {
        const children = [];
        this.indent += 1;
        children.push((0, Visitor_1.visit)(instructionDataArgs.struct, this));
        if (instructionDataArgs.link) {
            children.push((0, Visitor_1.visit)(instructionDataArgs.link, this));
        }
        this.indent -= 1;
        return [this.indented('[InstructionDataArgsNode]'), ...children].join('\n');
    }
    visitInstructionExtraArgs(instructionExtraArgs) {
        const children = [];
        this.indent += 1;
        children.push((0, Visitor_1.visit)(instructionExtraArgs.struct, this));
        if (instructionExtraArgs.link) {
            children.push((0, Visitor_1.visit)(instructionExtraArgs.link, this));
        }
        this.indent -= 1;
        return [this.indented('[InstructionExtraArgsNode]'), ...children].join('\n');
    }
    visitDefinedType(definedType) {
        this.indent += 1;
        const data = (0, Visitor_1.visit)(definedType.data, this);
        this.indent -= 1;
        return [this.indented(`[DefinedTypeNode] ${definedType.name}`), data].join('\n');
    }
    visitError(error) {
        return this.indented(`[ErrorNode] ${error.name} (${error.code}): ${error.message}`);
    }
    visitArrayType(arrayType) {
        this.indent += 1;
        const child = (0, Visitor_1.visit)(arrayType.child, this);
        this.indent -= 1;
        const size = (0, shared_1.displaySizeStrategy)(arrayType.size);
        return [this.indented(`[ArrayTypeNode] size: ${size}`), child].join('\n');
    }
    visitLinkType(linkType) {
        return this.indented(`[LinkTypeNode] ${linkType.name}, importFrom: ${linkType.importFrom}`);
    }
    visitEnumType(enumType) {
        this.indent += 1;
        const children = enumType.variants.map((variant) => (0, Visitor_1.visit)(variant, this));
        this.indent -= 1;
        return [this.indented('[EnumTypeNode]'), ...children].join('\n');
    }
    visitEnumEmptyVariantType(enumEmptyVariantType) {
        return this.indented(`[EnumEmptyVariantTypeNode] ${enumEmptyVariantType.name}`);
    }
    visitEnumStructVariantType(enumStructVariantType) {
        this.indent += 1;
        const child = (0, Visitor_1.visit)(enumStructVariantType.struct, this);
        this.indent -= 1;
        return [
            this.indented(`[EnumStructVariantTypeNode] ${enumStructVariantType.name}`),
            child,
        ].join('\n');
    }
    visitEnumTupleVariantType(enumTupleVariantType) {
        this.indent += 1;
        const child = (0, Visitor_1.visit)(enumTupleVariantType.tuple, this);
        this.indent -= 1;
        return [
            this.indented(`[EnumTupleVariantTypeNode] ${enumTupleVariantType.name}`),
            child,
        ].join('\n');
    }
    visitMapType(mapType) {
        const result = [];
        const size = (0, shared_1.displaySizeStrategy)(mapType.size);
        result.push(this.indented(`[MapTypeNode] size: ${size}, ${mapType.idlMap}`));
        this.indent += 1;
        result.push(this.indented('keys:'));
        this.indent += 1;
        result.push((0, Visitor_1.visit)(mapType.key, this));
        this.indent -= 1;
        result.push(this.indented('values:'));
        this.indent += 1;
        result.push((0, Visitor_1.visit)(mapType.value, this));
        this.indent -= 1;
        this.indent -= 1;
        return result.join('\n');
    }
    visitOptionType(optionType) {
        this.indent += 1;
        const child = (0, Visitor_1.visit)(optionType.child, this);
        this.indent -= 1;
        const prefix = optionType.prefix.toString();
        const fixed = optionType.fixed ? ', fixed' : '';
        return [
            this.indented(`[OptionTypeNode] prefix: ${prefix}${fixed}`),
            child,
        ].join('\n');
    }
    visitSetType(setType) {
        this.indent += 1;
        const child = (0, Visitor_1.visit)(setType.child, this);
        this.indent -= 1;
        const size = (0, shared_1.displaySizeStrategy)(setType.size);
        return [this.indented(`[SetTypeNode] size: ${size}`), child].join('\n');
    }
    visitStructType(structType) {
        this.indent += 1;
        const children = structType.fields.map((field) => (0, Visitor_1.visit)(field, this));
        this.indent -= 1;
        return [this.indented('[StructTypeNode]'), ...children].join('\n');
    }
    visitStructFieldType(structFieldType) {
        this.indent += 1;
        const child = (0, Visitor_1.visit)(structFieldType.child, this);
        this.indent -= 1;
        return [
            this.indented(`[StructFieldTypeNode] ${structFieldType.name}`),
            child,
        ].join('\n');
    }
    visitTupleType(tupleType) {
        this.indent += 1;
        const children = tupleType.children.map((child) => (0, Visitor_1.visit)(child, this));
        this.indent -= 1;
        return [this.indented('[TupleTypeNode]'), ...children].join('\n');
    }
    visitBoolType(boolType) {
        return this.indented(`[BoolTypeNode] ${boolType.size.toString()}`);
    }
    visitBytesType(bytesType) {
        return this.indented(`[BytesTypeNode] size: ${(0, shared_1.displaySizeStrategy)(bytesType.size)}`);
    }
    visitNumberType(numberType) {
        return this.indented(`[NumberTypeNode] ${numberType.toString()}`);
    }
    visitNumberWrapperType(numberWrapperType) {
        this.indent += 1;
        const item = (0, Visitor_1.visit)(numberWrapperType.number, this);
        this.indent -= 1;
        const { wrapper } = numberWrapperType;
        const base = `[NumberWrapperTypeNode] ${wrapper.kind}`;
        switch (wrapper.kind) {
            case 'Amount':
                return [
                    this.indented(`${base} ` +
                        `identifier: ${wrapper.identifier}, ` +
                        `decimals: ${wrapper.decimals}`),
                    item,
                ].join('\n');
            default:
                return [this.indented(`${base}`), item].join('\n');
        }
    }
    visitPublicKeyType() {
        return this.indented('[PublicKeyTypeNode]');
    }
    visitStringType(stringType) {
        return this.indented(`[StringTypeNode] ` +
            `encoding: ${stringType.encoding}, ` +
            `size: ${(0, shared_1.displaySizeStrategy)(stringType.size)}`);
    }
    indented(text) {
        return this.separator.repeat(this.indent) + text;
    }
}
exports.GetNodeTreeStringVisitor = GetNodeTreeStringVisitor;
//# sourceMappingURL=GetNodeTreeStringVisitor.js.map