"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeduplicateIdenticalDefinedTypesVisitor = void 0;
const nodes = __importStar(require("../../nodes"));
const aggregators_1 = require("../aggregators");
const BaseThrowVisitor_1 = require("../BaseThrowVisitor");
const Visitor_1 = require("../Visitor");
const DeleteNodesVisitor_1 = require("./DeleteNodesVisitor");
class DeduplicateIdenticalDefinedTypesVisitor extends BaseThrowVisitor_1.BaseThrowVisitor {
    visitRoot(root) {
        const typeMap = new Map();
        // Fill the type map with all defined types.
        root.programs.forEach((program) => {
            program.definedTypes.forEach((type) => {
                var _a;
                const typeWithProgram = { program, type };
                const list = (_a = typeMap.get(type.name)) !== null && _a !== void 0 ? _a : [];
                typeMap.set(type.name, [...list, typeWithProgram]);
            });
        });
        // Remove all types that are not duplicated.
        typeMap.forEach((list, name) => {
            if (list.length <= 1) {
                typeMap.delete(name);
            }
        });
        // Remove duplicates whose types are not equal.
        const strVisitor = new aggregators_1.GetNodeInlineStringVisitor();
        typeMap.forEach((list, name) => {
            const types = list.map((item) => (0, Visitor_1.visit)(item.type, strVisitor));
            const typesAreEqual = types.every((type, i, arr) => type === arr[0]);
            if (!typesAreEqual) {
                typeMap.delete(name);
            }
        });
        // Get the selectors for all defined types that needs deleting.
        // Thus, we must select all but the first duplicate of each list.
        const deleteSelectors = Array.from(typeMap.values())
            // Order lists by program index, get their tails and flatten.
            .flatMap((list) => {
            const sortedList = list.sort((a, b) => root.programs.indexOf(a.program) - root.programs.indexOf(b.program));
            const [, ...sortedListTail] = sortedList;
            return sortedListTail;
        })
            // Get selectors from the defined types and their programs.
            .map(({ program, type }) => ({
            kind: 'definedTypeNode',
            name: type.name,
            program: program.name,
        }));
        // Delete the identified nodes if any.
        if (deleteSelectors.length > 0) {
            const newRoot = (0, Visitor_1.visit)(root, new DeleteNodesVisitor_1.DeleteNodesVisitor(deleteSelectors));
            nodes.assertRootNode(newRoot);
            return newRoot;
        }
        return root;
    }
}
exports.DeduplicateIdenticalDefinedTypesVisitor = DeduplicateIdenticalDefinedTypesVisitor;
//# sourceMappingURL=DeduplicateIdenticalDefinedTypesVisitor.js.map