"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SetMissingDefinedTypesVisitor = void 0;
const nodes = __importStar(require("../../nodes"));
const aggregators_1 = require("../aggregators");
const BaseThrowVisitor_1 = require("../BaseThrowVisitor");
const Visitor_1 = require("../Visitor");
class SetMissingDefinedTypesVisitor extends BaseThrowVisitor_1.BaseThrowVisitor {
    constructor(programs) {
        super();
        this.programs = [];
        const root = nodes.rootNodeFromIdls(programs);
        this.programs = root.programs;
    }
    visitRoot(root) {
        // Get all linked defined types missing from the registered programs.
        const histogram = (0, Visitor_1.visit)(root, new aggregators_1.GetDefinedTypeHistogramVisitor());
        const availableTypes = nodes
            .getAllDefinedTypes(root)
            .map((type) => type.name);
        const missingTypes = Object.keys(histogram).filter((name) => {
            const { total } = histogram[name];
            return total > 0 && !availableTypes.includes(name);
        });
        // If no missing types, abort.
        if (missingTypes.length === 0) {
            return root;
        }
        // Get all programs that define the missing types
        // and trim them to only include the missing types.
        const foundTypes = new Set();
        const foundPrograms = this.programs.flatMap((program) => {
            const definedTypes = program.definedTypes.filter((type) => {
                if (foundTypes.has(type.name))
                    return false;
                const found = missingTypes.includes(type.name);
                if (found)
                    foundTypes.add(type.name);
                return found;
            });
            return definedTypes.length > 0
                ? [nodes.programNode({ ...program, definedTypes, internal: true })]
                : [];
        });
        // If no provided program includes missing types, abort.
        if (foundPrograms.length === 0) {
            return root;
        }
        // Merge the existing programs with the found programs.
        const newPrograms = [...root.programs];
        foundPrograms.forEach((foundProgram) => {
            const index = newPrograms.findIndex((newProgram) => newProgram.name === foundProgram.name);
            if (index === -1) {
                newPrograms.push(foundProgram);
                return;
            }
            const currentProgram = newPrograms[index];
            newPrograms[index] = nodes.programNode({
                ...currentProgram,
                definedTypes: [
                    ...currentProgram.definedTypes,
                    ...foundProgram.definedTypes,
                ],
            });
        });
        return nodes.rootNode(newPrograms);
    }
}
exports.SetMissingDefinedTypesVisitor = SetMissingDefinedTypesVisitor;
//# sourceMappingURL=SetMissingDefinedTypesVisitor.js.map