import type { ConfigureOptions } from 'nunjucks';
import { Options as PrettierOptions } from 'prettier';
import * as nodes from '../../nodes';
import { ImportFrom } from '../../shared';
import { BaseThrowVisitor, ResolvedInstructionAccount, ResolvedInstructionInput, Visitor } from '../../visitors';
import { RenderMap } from '../RenderMap';
import { JavaScriptTypeManifest } from './GetJavaScriptTypeManifestVisitor';
import { JavaScriptImportMap } from './JavaScriptImportMap';
export type GetJavaScriptRenderMapOptions = {
    renderParentInstructions?: boolean;
    formatCode?: boolean;
    prettierOptions?: PrettierOptions;
    dependencyMap?: Record<ImportFrom, string>;
    typeManifestVisitor?: Visitor<JavaScriptTypeManifest>;
    byteSizeVisitor?: Visitor<number | null> & {
        registerDefinedTypes?: (definedTypes: nodes.DefinedTypeNode[]) => void;
    };
    resolvedInstructionInputVisitor?: Visitor<ResolvedInstructionInput[]>;
};
export declare class GetJavaScriptRenderMapVisitor extends BaseThrowVisitor<RenderMap> {
    readonly options: Required<GetJavaScriptRenderMapOptions>;
    private program;
    constructor(options?: GetJavaScriptRenderMapOptions);
    visitRoot(root: nodes.RootNode): RenderMap;
    visitProgram(program: nodes.ProgramNode): RenderMap;
    visitAccount(account: nodes.AccountNode): RenderMap;
    visitInstruction(instruction: nodes.InstructionNode): RenderMap;
    visitDefinedType(definedType: nodes.DefinedTypeNode): RenderMap;
    get typeManifestVisitor(): Visitor<JavaScriptTypeManifest>;
    get byteSizeVisitor(): Visitor<number | null> & {
        registerDefinedTypes?: ((definedTypes: nodes.DefinedTypeNode[]) => void) | undefined;
    };
    get resolvedInstructionInputVisitor(): Visitor<ResolvedInstructionInput[]>;
    protected getInstructionAccountType(account: ResolvedInstructionAccount): string;
    protected getInstructionAccountImports(accounts: ResolvedInstructionAccount[]): JavaScriptImportMap;
    protected getMergeConflictsForInstructionAccountsAndArgs(instruction: nodes.InstructionNode): string[];
    protected render(template: string, context?: object, options?: ConfigureOptions): string;
}
