import * as nodes from '../../nodes';
import { Visitor } from '../../visitors';
import { JavaScriptImportMap } from './JavaScriptImportMap';
export type JavaScriptTypeManifest = {
    isEnum: boolean;
    strictType: string;
    strictImports: JavaScriptImportMap;
    looseType: string;
    looseImports: JavaScriptImportMap;
    serializer: string;
    serializerImports: JavaScriptImportMap;
};
export declare class GetJavaScriptTypeManifestVisitor implements Visitor<JavaScriptTypeManifest> {
    readonly serializerVariable: string;
    private parentName;
    constructor(serializerVariable?: string);
    visitRoot(): JavaScriptTypeManifest;
    visitProgram(): JavaScriptTypeManifest;
    visitAccount(account: nodes.AccountNode): JavaScriptTypeManifest;
    visitAccountData(accountData: nodes.AccountDataNode): JavaScriptTypeManifest;
    visitInstruction(instruction: nodes.InstructionNode): JavaScriptTypeManifest;
    visitInstructionAccount(): JavaScriptTypeManifest;
    visitInstructionDataArgs(instructionDataArgs: nodes.InstructionDataArgsNode): JavaScriptTypeManifest;
    visitInstructionExtraArgs(instructionExtraArgs: nodes.InstructionExtraArgsNode): JavaScriptTypeManifest;
    visitDefinedType(definedType: nodes.DefinedTypeNode): JavaScriptTypeManifest;
    visitError(): JavaScriptTypeManifest;
    visitArrayType(arrayType: nodes.ArrayTypeNode): JavaScriptTypeManifest;
    visitLinkType(linkType: nodes.LinkTypeNode): JavaScriptTypeManifest;
    visitEnumType(enumType: nodes.EnumTypeNode): JavaScriptTypeManifest;
    visitEnumEmptyVariantType(enumEmptyVariantType: nodes.EnumEmptyVariantTypeNode): JavaScriptTypeManifest;
    visitEnumStructVariantType(enumStructVariantType: nodes.EnumStructVariantTypeNode): JavaScriptTypeManifest;
    visitEnumTupleVariantType(enumTupleVariantType: nodes.EnumTupleVariantTypeNode): JavaScriptTypeManifest;
    visitMapType(mapType: nodes.MapTypeNode): JavaScriptTypeManifest;
    visitOptionType(optionType: nodes.OptionTypeNode): JavaScriptTypeManifest;
    visitSetType(setType: nodes.SetTypeNode): JavaScriptTypeManifest;
    visitStructType(structType: nodes.StructTypeNode): JavaScriptTypeManifest;
    visitStructFieldType(structFieldType: nodes.StructFieldTypeNode): JavaScriptTypeManifest;
    visitTupleType(tupleType: nodes.TupleTypeNode): JavaScriptTypeManifest;
    visitBoolType(boolType: nodes.BoolTypeNode): JavaScriptTypeManifest;
    visitBytesType(bytesType: nodes.BytesTypeNode): JavaScriptTypeManifest;
    visitNumberType(numberType: nodes.NumberTypeNode): JavaScriptTypeManifest;
    visitNumberWrapperType(numberWrapperType: nodes.NumberWrapperTypeNode): JavaScriptTypeManifest;
    visitPublicKeyType(): JavaScriptTypeManifest;
    visitStringType(stringType: nodes.StringTypeNode): JavaScriptTypeManifest;
    protected mergeManifests(manifests: JavaScriptTypeManifest[]): Pick<JavaScriptTypeManifest, 'strictImports' | 'looseImports' | 'serializerImports' | 'isEnum'>;
    protected createDocblock(docs: string[]): string;
    protected getArrayLikeSizeOption(size: nodes.ArrayTypeNode['size'], manifest: Pick<JavaScriptTypeManifest, 'strictImports' | 'looseImports' | 'serializerImports'>): string | null;
}
