import { AccountNode, ValueNode } from '../nodes';
import { ImportFrom } from './ImportFrom';
export type InstructionDefault = {
    kind: 'identity';
} | {
    kind: 'payer';
} | {
    kind: 'programId';
} | {
    kind: 'program';
    program: {
        name: string;
        publicKey: string;
    };
} | {
    kind: 'publicKey';
    publicKey: string;
} | {
    kind: 'account';
    name: string;
} | {
    kind: 'accountBump';
    name: string;
} | {
    kind: 'arg';
    name: string;
} | {
    kind: 'value';
    value: ValueNode;
} | {
    kind: 'pda';
    pdaAccount: string;
    importFrom: ImportFrom;
    seeds: Record<string, InstructionSeedDefault>;
} | {
    kind: 'resolver';
    name: string;
    importFrom: ImportFrom;
    dependsOn: InstructionDependency[];
    resolvedIsSigner?: boolean | 'either';
    resolvedIsOptional?: boolean;
};
export type ExtractInstructionDefault<T extends InstructionDefault['kind']> = Extract<InstructionDefault, {
    kind: T;
}>;
export type InstructionAccountDefault = ExtractInstructionDefault<'programId' | 'program' | 'publicKey' | 'account' | 'identity' | 'payer' | 'pda' | 'resolver'>;
export type InstructionArgDefault = InstructionDefault;
export type InstructionSeedDefault = ExtractInstructionDefault<'arg' | 'account' | 'value'>;
export declare const identityDefault: () => ExtractInstructionDefault<'identity'>;
export declare const payerDefault: () => ExtractInstructionDefault<'payer'>;
export declare const programIdDefault: () => ExtractInstructionDefault<'programId'>;
export declare const programDefault: (name: string, publicKey: string) => ExtractInstructionDefault<'program'>;
export declare const publicKeyDefault: (publicKey: string) => ExtractInstructionDefault<'publicKey'>;
export declare const accountDefault: (account: string) => ExtractInstructionDefault<'account'>;
export declare const accountBumpDefault: (account: string) => ExtractInstructionDefault<'accountBump'>;
export declare const argDefault: (arg: string) => ExtractInstructionDefault<'arg'>;
export declare const valueDefault: (value: ValueNode) => ExtractInstructionDefault<'value'>;
export declare const pdaDefault: (pdaAccount: string, options?: {
    importFrom?: ImportFrom;
    seeds?: Record<string, InstructionSeedDefault>;
}) => ExtractInstructionDefault<'pda'>;
export declare const resolverDefault: (name: string, dependsOn: InstructionDependency[], options?: {
    importFrom?: ImportFrom;
    resolvedIsSigner?: boolean | 'either';
    resolvedIsOptional?: boolean;
}) => ExtractInstructionDefault<'resolver'>;
export type InstructionDependency = {
    kind: 'account' | 'arg';
    name: string;
};
export declare const dependsOnAccount: (account: string) => InstructionDependency;
export declare const dependsOnArg: (arg: string) => InstructionDependency;
export declare const getDefaultSeedsFromAccount: (node: AccountNode) => Record<string, InstructionSeedDefault>;
